@@ -16,7 +16,6 @@ NarrowPeaks files.
 import math
 from copy import deepcopy
 import multiprocessing as mp
-from scipy.stats import rankdata
 from scipy.stats import norm
 from scipy.stats import multivariate_normal
 from scipy.stats import bernoulli
@@ -24,6 +23,7 @@ from scipy.optimize import brentq
 import numpy as np
 import pandas as pd
 import midr.log as log
+from midr.auxiliary import compute_empirical_marginal_cdf, compute_rank
 
 
 def cov_matrix(m_sample, theta):
@@ -105,52 +105,6 @@ def sim_m_samples(n_value, m_sample,
     return {'X': scores, 'K': k_state}
 
 
-def compute_rank(x_score):
-    """
-    transform x a n*m matrix of score into an n*m matrix of rank ordered by
-    row.
-
-    >>> compute_rank(np.array([[0,0],[10,30],[20,20],[30,10]]))
-    array([[1, 1],
-           [2, 4],
-           [3, 3],
-           [4, 2]])
-    """
-    rank = np.empty_like(x_score)
-    for j in range(x_score.shape[1]):
-        # we want the rank to start at 1
-        rank[:, j] = rankdata(x_score[:, j], method="ordinal")
-    return rank
-
-
-def compute_empirical_marginal_cdf(rank):
-    """
-    normalize ranks to compute empirical marginal cdf and scale by n / (n+1)
-
-    >>> r = compute_rank(np.array(
-    ...    [[0.0,0.0],
-    ...    [10.0,30.0],
-    ...    [20.0,20.0],
-    ...    [30.0,10.0]]))
-    >>> compute_empirical_marginal_cdf(r)
-    array([[0.99  , 0.99  ],
-           [0.7425, 0.2475],
-           [0.495 , 0.495 ],
-           [0.2475, 0.7425]])
-    """
-    x_score = np.empty_like(rank)
-    n_value = float(rank.shape[0])
-    m_sample = float(rank.shape[1])
-    # scaling_factor = n_value / (n_value + 1.0)
-    # we want a max value of 0.99
-    scaling_factor = 0.99
-    for i in range(int(n_value)):
-        for j in range(int(m_sample)):
-            x_score[i][j] = (1.0 - (float(rank[i][j] - 1) / n_value)) * \
-                            scaling_factor
-    return x_score
-
-
 def g_function(z_values, theta):
     """
     compute scalded Gaussian cdf for Copula
