@@ -18,7 +18,8 @@ from scipy.optimize import minimize
 import numpy as np
 import midr.log as log
 import midr.archimedean as archimedean
-from midr.idr import compute_empirical_marginal_cdf, compute_rank, sim_m_samples
+from midr.auxiliary import compute_empirical_marginal_cdf, compute_rank
+from midr.idr import sim_m_samples
 import midr.archimediean_plots as archimediean_plots
 
 COPULA_DENSITY = {
@@ -32,6 +33,170 @@ DMLE_COPULA = {
     'gumbel': archimedean.dmle_copula_gumbel
 }
 
+def non_zero_in_row(u_values, ncol=0):
+    """
+    return an array of bool on the presence of zeros in rows
+    :param u_values:
+    :param ncol:
+    :return:
+    >>> non_zero_in_row(np.array([
+    ...    [0.42873569, 0.18285458, 0.9514195],
+    ...    [0.25148149, 0.0, 0.3378213],
+    ...    [0.79410993, 0.76175687, 0.0709562],
+    ...    [0.02694249, 0.45788802, 0.6299574],
+    ...    [0.39522060, 0.0, 0.0],
+    ...    [0.66878367, 0.38075101, 0.5185625],
+    ...    [0.90365653, 0.19654621, 0.6809525],
+    ...    [0.0, 0.82713755, 0.7686878],
+    ...    [0.22437343, 0.16907646, 0.5740400],
+    ...    [0.66752741, 0.69487362, 0.3329266]
+    ...    ]),
+    ...    ncol=0
+    ... )
+    array([0, 2, 3, 5, 6, 8, 9])
+    """
+    return np.where(~(np.sum(u_values == 0.0, axis=1) > ncol))[0]
+
+
+def censored_u_values(u_values, ncol=0):
+    """
+    return u_values with ncol without zeros
+    :param u_values:
+    :param ncol:
+    :return:
+    >>> u_values = np.array([
+    ...    [0.42873569, 0.18285458, 0.9514195],
+    ...    [0.25148149, 0.0, 0.3378213],
+    ...    [0.79410993, 0.76175687, 0.0709562],
+    ...    [0.02694249, 0.45788802, 0.6299574],
+    ...    [0.39522060, 0.0, 0.0],
+    ...    [0.66878367, 0.38075101, 0.5185625],
+    ...    [0.90365653, 0.19654621, 0.6809525],
+    ...    [0.0, 0.82713755, 0.7686878],
+    ...    [0.22437343, 0.16907646, 0.5740400],
+    ...    [0.66752741, 0.69487362, 0.3329266]
+    ...    ])
+    >>> censored_u_values(u_values,
+    ... ncol=0
+    ... )
+    [array([0, 2, 3, 5, 6, 8, 9]), array([[0.42873569, 0.18285458, 0.9514195 ],
+           [0.79410993, 0.76175687, 0.0709562 ],
+           [0.02694249, 0.45788802, 0.6299574 ],
+           [0.66878367, 0.38075101, 0.5185625 ],
+           [0.90365653, 0.19654621, 0.6809525 ],
+           [0.22437343, 0.16907646, 0.57404   ],
+           [0.66752741, 0.69487362, 0.3329266 ]])]
+    >>> censored_u_values(u_values,
+    ... ncol=1
+    ... )
+    [array([0, 1, 2, 3, 5, 6, 7, 8, 9]), array([[0.42873569, 0.18285458, 0.9514195 ],
+           [0.25148149, 0.        , 0.3378213 ],
+           [0.79410993, 0.76175687, 0.0709562 ],
+           [0.02694249, 0.45788802, 0.6299574 ],
+           [0.66878367, 0.38075101, 0.5185625 ],
+           [0.90365653, 0.19654621, 0.6809525 ],
+           [0.        , 0.82713755, 0.7686878 ],
+           [0.22437343, 0.16907646, 0.57404   ],
+           [0.66752741, 0.69487362, 0.3329266 ]])]
+    """
+    censored_row = non_zero_in_row(u_values=u_values, ncol=ncol)
+    return [censored_row, u_values[censored_row, ]]
+
+
+def list_censored_u_values(u_values):
+    """
+    return u_values with ncol without zeros
+    :param u_values:
+    :param ncol:
+    :return:
+    >>> u_values = np.array([
+    ...    [0.42873569, 0.18285458, 0.9514195],
+    ...    [0.25148149, 0.0, 0.3378213],
+    ...    [0.79410993, 0.76175687, 0.0709562],
+    ...    [0.02694249, 0.45788802, 0.6299574],
+    ...    [0.39522060, 0.0, 0.0],
+    ...    [0.66878367, 0.38075101, 0.5185625],
+    ...    [0.90365653, 0.19654621, 0.6809525],
+    ...    [0.0, 0.82713755, 0.7686878],
+    ...    [0.22437343, 0.16907646, 0.5740400],
+    ...    [0.66752741, 0.69487362, 0.3329266]
+    ...    ])
+    >>> list_censored_u_values(u_values)
+    [[array([0, 2, 3, 5, 6, 8, 9]), array([[0.42873569, 0.18285458, 0.9514195 ],
+           [0.79410993, 0.76175687, 0.0709562 ],
+           [0.02694249, 0.45788802, 0.6299574 ],
+           [0.66878367, 0.38075101, 0.5185625 ],
+           [0.90365653, 0.19654621, 0.6809525 ],
+           [0.22437343, 0.16907646, 0.57404   ],
+           [0.66752741, 0.69487362, 0.3329266 ]])], [array([0, 2, 3, 5, 6, 7, 8, 9]), array([[0.18285458, 0.9514195 ],
+           [0.76175687, 0.0709562 ],
+           [0.45788802, 0.6299574 ],
+           [0.38075101, 0.5185625 ],
+           [0.19654621, 0.6809525 ],
+           [0.82713755, 0.7686878 ],
+           [0.16907646, 0.57404   ],
+           [0.69487362, 0.3329266 ]])], [array([0, 1, 2, 3, 5, 6, 8, 9]), array([[0.42873569, 0.9514195 ],
+           [0.25148149, 0.3378213 ],
+           [0.79410993, 0.0709562 ],
+           [0.02694249, 0.6299574 ],
+           [0.66878367, 0.5185625 ],
+           [0.90365653, 0.6809525 ],
+           [0.22437343, 0.57404   ],
+           [0.66752741, 0.3329266 ]])], [array([0, 2, 3, 5, 6, 8, 9]), array([[0.42873569, 0.18285458],
+           [0.79410993, 0.76175687],
+           [0.02694249, 0.45788802],
+           [0.66878367, 0.38075101],
+           [0.90365653, 0.19654621],
+           [0.22437343, 0.16907646],
+           [0.66752741, 0.69487362]])]]
+    """
+    censored_data = [censored_u_values(u_values=u_values)]
+    for i in range(u_values.shape[1]):
+        for j in range(u_values.shape[1]):
+            minus_i = ~(i == np.array(range(u_values.shape[1])))
+            censored_data.append(
+                censored_u_values(
+                    u_values=u_values[:, minus_i],
+                    ncol=j
+                )
+            )
+    return censored_data
+
+
+def censored_copula_density(u_values, theta, copula):
+    """
+    compute pdf of copula for censored data
+    :param u_values:
+    :param theta:
+    :param copula:
+    :return:
+    >>> censored_copula_density(np.array([
+    ...    [0.42873569, 0.18285458, 0.9514195, 0.9514195],
+    ...    [0.25148149, 0.0, 0.3378213, 0.3378213],
+    ...    [0.79410993, 0.76175687, 0.0709562, 0.0709562],
+    ...    [0.02694249, 0.45788802, 0.6299574, 0.6299574],
+    ...    [0.39522060, 0.0, 0.0, 0.6299574],
+    ...    [0.66878367, 0.38075101, 0.5185625, 0.5185625],
+    ...    [0.90365653, 0.19654621, 0.6809525, 0.6809525],
+    ...    [0.0, 0.82713755, 0.7686878, 0.7686878],
+    ...    [0.22437343, 0.16907646, 0.5740400, 0.5740400],
+    ...    [0.66752741, 0.69487362, 0.3329266, 0.3329266]
+    ...    ]),
+    ...    theta=6,
+    ...    copula="frank",
+    ... )
+    """
+    censored_datas = list_censored_u_values(u_values=u_values)
+    dcopula = np.zeros((u_values.shape[0], len(censored_datas)))
+    i = 0
+    for censored_data in censored_datas:
+        dcopula[censored_data[0], i] = COPULA_DENSITY[copula](
+            u_values=censored_data[1],
+            theta=theta
+        )
+        i += 1
+    return dcopula
+
 
 def delta(params_list, threshold):
     """
